/*
 * calc_cascade_input_voltage.c
 *
 *  Created on: Dec 2, 2020
 *      Author: a0873438
 */
#include "cascade_input_voltage.h"
#include <msp430.h>

#include <stdint.h>
//#include <stdbool.h>
//#include "time_delay.h"

/**** Local Function Prototypes ***********************************************/

static inline uint16_t read_v_out2(void);
static inline uint32_t calc_cascade_input_voltage(uint16_t);

/**** Local Variables ********************************************************/

//uint32_t gained_input_voltage_value = 0;    // Calculated G1*Vin1(n)
//static uint32_t ema_output_value_yn;        // y(n) <= EMA(G1*Vin1(n))

/**** Functions **************************************************************/

/*!
 *
 */

uint32_t cascade_input_voltage (void)
{
    uint16_t v_out2;
    uint32_t v_in1;

    v_out2 = read_v_out2();
    v_in1 = calc_cascade_input_voltage(v_out2);

    return (v_in1);

}   /* calc_cascade_input_voltage() */



/*! \brief Calculates G1*G2*Vin1
 *
 *  G1*G2*Vin1 = Vout2 - (1-G1)*G2*Vdac1 - (1-G2)*Vdac2
 *
 *  \return G1*G2*Vin1
 */
static inline uint32_t
calc_cascade_input_voltage (uint16_t v_out2_value)
{
    uint16_t v_dac1_value;
    uint16_t v_dac2_value;
    uint32_t result;

    v_dac1_value = SAC3DAT;
    v_dac2_value = SAC1DAT;

    /*
     * -G1*G2*Vin1 = -Vout2 + (1-G1)*G2*Vdac1 + (1-G2)*Vdac2
     * -G1*G2*Vin1 = -Vout2 + M1*Vdac1 + M2*Vdac2
     * where:
     *      M1 = (1-G1)*G2 = 2^8
     *      M2 = (1-G2)    = 2^4 + 1
     * for:
     *      G1 = 17
     *      G2 = -16
     */
#   define M1_SHIFT     8       // Shift applied for (1-G1)*G2 multiplication
#   define M2_SHIFT     4       // Shift applied for (1-G2) multiplication

    result = (uint32_t) v_dac1_value << M1_SHIFT;       // (1-G1)*G2*Vdac1
    result += (uint32_t) v_dac2_value << M2_SHIFT;      //
    result += (uint32_t) v_dac2_value;                  // +(1-G2)*Vdac2
    result -= (uint32_t) v_out2_value;                  // -Vout2

    return (result);

}   /* calc_cascade_input_voltage() */


static inline uint16_t
read_v_out2 (void)
{
    SAC1DAC |= DACEN;       // Enable SAC1 DAC.
    SAC1OA |= (OAEN  |      // Enable SAC1 OA.
               SACEN        // Enable SAC1.
              );            // this one requires maybe 5us delay before ADC starts

    // ADC Channel A4 (Vour2 signal)
    //
    ADCCTL0 &= ~ADCENC;                 // To modify control bits must disable ADC.
    ADCMCTL0 = ADCINCH_4 | ADCSREF_1;   // Configure ADC to sample channel A4 with
                                        // V(R+) = VREF and V(R-) = AVSS
    ADCCTL0 |= ADCENC | ADCSC;          // Sampling and conversion start
    __low_power_mode_0();               // Returning here from ADC ISR in AM

    return (ADCMEM0);

}   /* read_v_out2() */


/*** end of file ***/
