/*
 * adc_config.c
 *
 *  Created on: Jun 7, 2020
 *      Author: a0873438
 */

#include "adc.h"
#include "adc_config.h"

/**** Functions **************************************************************/

//! \brief Initializes the ADC
//!
//! \return none
void
hal_adc_init (void)
{
#if (0) // Manual enable or disable
    ADC_init(ADC_BASE,
             ADC_SAMPLEHOLDSOURCE_SC,   // ADC sample-and-hold source select, ADCSC bit
             ADC_CLOCKSOURCE_SMCLK,     // ADC clock source select, SMCLK (8 MHz)
             ADC_CLOCKDIVIDER_2         // Divide by 2, ADCCLK = 4 MHz
            );

    ADCCTL1 |= ADCSSEL_3;

    ADC_setupSamplingTimer(ADC_BASE,
                           ADC_CYCLEHOLD_128_CYCLES,    // ADC sample-and-hold time, 128 ADCCLK cycles
                           ADC_MULTIPLESAMPLESDISABLE   // Single conversion, trigger will be needed to start every ADC conversion.
                          );

    // ADC sample-and-hold time, 128 / 4 MHz = 32 us
    // total (128 + 14) / 4 MHz = 35.5 us (28 kSPS max)
    // this configuration gave ~37  us (logic probe) sampling and conversion time

    ADC_enable(ADC_BASE);        // Enable ADC core

    // 12 bit (14 clock cycle conversion time)
    ADC_setResolution(ADC_BASE,ADC_RESOLUTION_12BIT);

    // ADC data read-back format, binary unsigned
    ADC_setDataReadBackFormat(ADC_BASE,ADC_UNSIGNED_BINARY);

    // Sampling rate < 50 kSPS
    ADC_setReferenceBufferSamplingRate(ADC_BASE,ADC_MAXSAMPLINGRATE_50KSPS);

    // Enable the interrupt request for a completed ADC conversion.
    ADC_enableInterrupt(ADC_BASE,ADC_COMPLETED_INTERRUPT);
#endif

#if (1) // Manual enable or disable
    // Initialize ADC
    ADCCTL0 = ADCSHT_6 |                    // ADC sample-and-hold time, 128 ADCCLK cycles (32 us); Single conversion
              ADCON;                        // Enable ADC core

    ADCCTL1 = ADCSHP |                      // Pulse sample mode; Software trigger
              ADCCONSEQ_0 |                 // Single-channel single-conversion
              ADCSSEL_3 | ADCDIV_1;         // ADCCLK = SMCLK (8 MHz); Divide by 2

    ADCCTL2 = ADCRES_2; //|                    // 12-bit conversion results, binary unsigned
              // ADCSR;                        // Sampling rate < 50ksps (20 kHz), not sure if needed for internal 2.0 V

    ADCIE |= ADCIE0;                        // Enable ADC-conv-complete interrupt

}   /* hal_adc_init() */
#endif


//! \brief Enables the ADC
//!
//! \return none
void
hal_adc_enable (void)
{
    // Clear Interrupt, the ADCIFG0 is set when an ADC conversion is completed.
    ADC_clearInterrupt(ADC_BASE,ADC_COMPLETED_INTERRUPT_FLAG);

    // Enable interrupt, this bit enable the interrupt request for a completed ADC conversion.
    ADC_enable(ADC_BASE);

    // NOTE: ADC will be turned on and started by timer
    //
}   /* hal_adc_enable */


//! \brief Stops and disables the ADC
//!
//! \return none
void
hal_adc_disable (void)
{
    // Disable ADC interrupt, this bit disable the interrupt request for a completed ADC conversion.
    ADC_disableInterrupt(ADC_BASE,ADC_COMPLETED_INTERRUPT);

    // Disables the ADC from converting any more signals.
    // Allows the ADC to end the current conversion before disabling conversions.
    ADC_disableConversions(ADC_BASE,ADC_COMPLETECONVERSION);

    // Clear Interrupt, the ADCIFG0 is set when an ADC conversion is completed.
    ADC_clearInterrupt(ADC_BASE,ADC_COMPLETED_INTERRUPT_FLAG);

}   /* hal_adc_disable() */

/*** end of file ***/
