/*
 * bsp.h
 *
 *  Created on: Nov 18, 2020
 *      Author: Miroslav Oljaca
 */

// Board Support Package (BSP)
// is part of Hardware Abstraction Layer (HAL). The primary objective of
// the BSP module is to provide one access point for users to modify the
// hardware mapping details without requiring an extensive search and replace
// operation. In order to achieve this objective, the BSP is divided into two
// inter-dependent interfaces: Application Interface and Peripheral Interface.

#ifndef SOURCE_MSP430PIR_HAL_BSP_H_
#define SOURCE_MSP430PIR_HAL_BSP_H_

#   include "gpio.h"

#   define ENABLE_GUI_COMM                  (0)     // Enable eUSCI communication with GUI Composer.
#   define ENABLE_TIMING_BENCHMARK_TPs      (0)     // Enable TP20 and TP40 for time measurements and debugging

// Define Test Points (TP), available on ISE4075A
//
#   define TP_P21_High     P2OUT |= BIT1   // P2.1
#   define TP_P21_Low      P2OUT &= ~BIT1

#   define TP_P40_High     P4OUT |= BIT0   // P4.0
#   define TP_P40_Low      P4OUT &= ~BIT0

// Define on-board LED status, available on ISE4075A
//
#   define Red_LED_off     P4OUT |= BIT0   // P4.0
#   define Red_LED_on      P4OUT &= ~BIT0

#   define Green_LED_off   P2OUT |= BIT1   // P2.1
#   define Green_LED_on    P2OUT &= ~BIT1

#endif /* SOURCE_MSP430PIR_HAL_BSP_H_ */

/*** end of file ***/
