/*
 * vref_config.c
 *
 *  Created on: Jun 6, 2020
 *      Author: a0873438
 */

#include "pmm.h"
#include "vref_config.h"

/**** Local Functions *********************************************************/

//! \brief Initializes the Internal VREF to 2.0 V
//!
//! \return none

void
hal_vref_init (void)
{
    PMM_enableInternalReference();                      // Unlock the PMM register
                                                        // Enable internal reference

    PMM_selectVoltageReference(PMM_REFVSEL_2_0V);       // Set internal reference to 2.0V

                                                        // Power Management Module Control Register 2, PMMCTL2
    while((PMM_isBandgapActive()                 |      // Reference bandgap active status
           PMM_getBufferedBandgapVoltageStatus() |      // Buffered bandgap voltage ready status
           PMM_isRefGenActive()                  |      // Reference generator active status
           PMM_getVariableReferenceVoltageStatus())     // Variable reference voltage ready status
          !=
          (PMM_REFBG_ACTIVE  |                          // 1b = Reference bandgap buffer active
           PMM_REFBG_READY   |                          // 1b = Buffered bandgap voltage is ready to be used
           PMM_REFGEN_ACTIVE |                          // 1b = Reference generator active
           PMM_REFGEN_READY))                           // 1b = Reference voltage output is ready to be used
    ;   // Empty loop - Wait till bandgap and variable reference are active and ready to use

}   /* hal_vref_int() */

/*** end of file ***/
