/* --COPYRIGHT--,BSD
 * Copyright (c) 2020, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//*****************************************************************************
/*
 * main.c
 *
 * Created on: Dec 12, 2020
 *     Author: Miroslav Oljaca
 */
#include <msp430.h>

#include "bsp.h"
#include "system_init.h"
#include "afe_init.h"
#include "timer_config.h"

#include "time_delay.h"

#include "remove_pir_dc_voltage.h"
#include "cascade_input_voltage.h"
#include "saturation_control.h"
#include "motion_detection.h"
#include "despiking_derippling.h"



/**** Global Variable Extern Declarations **************************************/
//
uint16_t adc_channel[16] = {0};       // ADC input channels form 0 to 15 are stored
int16_t first_gain_stage_offset = 0;    // Offset of the first gain stage
int32_t afe_raw_input_signal = 0;

uint16_t pir_dc_voltage;

#pragma PERSISTENT( SYSRSTIV_buffer )
//#pragma location = 0x4400;
uint32_t SYSRSTIV_buffer[32] = {0};

#pragma PERSISTENT( SYSRSTIV_buffer_ptr )
//#pragma location = 0x4400;
uint32_t SYSRSTIV_buffer_ptr = 0;


//Buffers for Datalogging of PIR Signal
#pragma PERSISTENT( PIR_Despiked_Data_buffer )
uint32_t PIR_Despiked_Data_buffer[PIR_DATALOGGING_LENGTH] = {0};
//#pragma PERSISTENT( PIR_Cascade_Gained_Input_Data_buffer )
//uint32_t PIR_Cascade_Gained_Input_Data_buffer[PIR_DATALOGGING_LENGTH] = {0};
#pragma PERSISTENT( PIR_Inst_Detn_th_val )
uint32_t PIR_Inst_Detn_th_val[PIR_DATALOGGING_LENGTH] = {0};
#pragma PERSISTENT( PIR_abs_inst_dev_value )
uint32_t PIR_abs_inst_dev_value[PIR_DATALOGGING_LENGTH] = {0};
#pragma PERSISTENT( PIR_motion_detection_logging )
uint32_t PIR_motion_detection_logging[PIR_DATALOGGING_LENGTH] = {0};

#pragma PERSISTENT( PIR_Data_pointer )
unsigned int PIR_Data_pointer = 0;
//**********************************




/**** Local Structure Types ***************************************************/

/**** Local Function Prototypes ***********************************************/


void
main (void)
{
    WDTCTL = WDTPW | WDTHOLD;	// stop watchdog timer



/**** Beginning of hardware initialization ***********************************/

    // Initialize GPIOs and Clock System
    //
    system_init();

    Red_LED_on;     // System initialized but not ready


/**** Reset Testing for Legrand ***********************************/
    //Reset Testing for Legrand
    //Store reset vector in FRAM.  Technically, there could be multiple reset sources FIFO'd, so will continue reading sources until it's 0.
    Green_LED_on;  // Added Green LED for Reset only.


    // Configure MPU too allow datalogging
    SYSCFG0 = FRWPPW | FRWPOA5 | FRWPOA4 | FRWPOA3 | FRWPOA2 | FRWPOA1 | FRWPOA0 | DFWP | PFWP;  // Configure up to 9kB bytes for FRAM datalogging

    //Store latest reset source in Non-volitile FRAM
    SYSRSTIV_buffer[SYSRSTIV_buffer_ptr++]=SYSRSTIV;

    if(SYSRSTIV_buffer[SYSRSTIV_buffer_ptr-1] != SYSRSTIV__RSTNMI){
        PIR_Data_pointer = 0;  // If anything but the RST pin, reset the datalogging.  (want to keep monitoring for RST pin resets.)
    }

    if(SYSRSTIV_buffer_ptr == 64){
        SYSRSTIV_buffer_ptr =0;
    }


//    SYSCFG0 = FRWPPW | DFWP | PFWP;     // Program FRAM write protected (not writable)
/*****************************************************************/


    // Initialize AFE
    //
    hal_afe_init();         // Initialize Vref
                            // Initialize ADC
                            // Initialize SAC

    // Initialize Timers
    //
    hal_timers_init ();    // TB0 generate 20 sps ISR
                           // TB1 generate 153 us delay ISR

/**** End of hardware initialization *****************************************/

//    Green_LED_on;

    uint32_t gained_cascade_input_voltage;
    uint32_t depeak_input_voltage;

/**** Start of software initialization ***************************************/

    remove_pir_dc_voltage();
    cascade_input_voltage();
    out_sig_sat_ctrl(adc_channel[4]);

/**** End of software initialization *****************************************/



    hal_timer_b0_start();
    __enable_interrupt();

    Green_LED_off;
    Red_LED_off;

    while (1)
    {
        __low_power_mode_3();   // Returning here from TB0.0 ISR in AM

        __disable_interrupt();
        remove_pir_dc_voltage();
        __enable_interrupt();

        gained_cascade_input_voltage = cascade_input_voltage();

//#ifdef DATALOGGING_ENABLE
//        //Logging the Gained Cascade Input Voltage
//        PIR_Cascade_Gained_Input_Data_buffer[PIR_Data_pointer] = gained_cascade_input_voltage;  // Just testing the datalogging
//#endif

        hal_afe_disable();      // Disable Analog

        depeak_input_voltage = despiking_derippling_calc(gained_cascade_input_voltage);

#ifdef DATALOGGING_ENABLE
        //Logging the Gained Cascade Input Voltage
        PIR_Despiked_Data_buffer[PIR_Data_pointer] = depeak_input_voltage;  // Just testing the datalogging
#endif

        signal_analysis(depeak_input_voltage);

        out_sig_sat_ctrl(adc_channel[4]);

#ifdef DATALOGGING_ENABLE
        PIR_Data_pointer += 1;
        __no_operation();
//        __NO_OPERATION;
        if(PIR_Data_pointer == PIR_DATALOGGING_LENGTH){PIR_Data_pointer = PIR_DATALOGGING_LENGTH-1;}  // Not Looping
        __no_operation();
#endif

    }

/* ----------------------------------------------------------------------------
 *
 * END of the NEW main LOOP
 *
 */



}   /* main() */

/*** end of file ***/
